import greenfoot.*;  // （World、Actor、GreenfootImage、GreenfootSound、Greenfoot及MouseInfo）
import java.awt.Point;

import java.util.ArrayList;
import java.util.List;

/**
 * 棋盘类，提供下棋的场所
 */
public class Board extends World {
    private ChessSide currentSide = null;  //目前轮到哪一方下棋
    private Score score = null;    //游戏分数
    public static ArrayList<Integer> soreList=new ArrayList<Integer> ();
    public static boolean gameover = false;
    //构造方法，初始化棋盘
    public Board() {
        super(9, 10, 50); 
        GreenfootImage backIm = getBackground();
        backIm.setColor(Color.BLACK);
        for(int i=0; i<10; i++){
            backIm.drawLine(0, 50 * i + 25, 450, 50 * i + 25 );
        }
        for(int i=0; i<9; i++){
            backIm.drawLine(50 * i + 25, 0, 50 * i + 25, 500);
        }
        setBackground(backIm);
        currentSide = ChessSide.RED;
        gameover = false;
        initBoard();
        score = new Score();
        addObject(score, 4, 9);  //添加游戏分数
        addObject(new Title(), 4, 3);  
        setPaintOrder(ScoreBoard.class,Hint.class,Piece.class);
    }

    public void started(){
        removeObjects(getObjects(Title.class));
    }
    
    //游戏结束时的操作
    public void gameOver() {
        updateRank(5);      //更新排行榜
        addObject(new ScoreBoard(soreList), 4, 4); //在程序窗口中央显示记分牌
        Greenfoot.playSound("fail.wav");
        //Greenfoot.stop();  //游戏停止
    }

    //更新排行榜，加入新的记录
    public void updateRank(int n){
        int time = score.getScore();
        //若当前记录大于列表中的某个记录，则将其插入到该记录之前
        for(int i = 0;i < soreList.size();i++){  
            if(time > soreList.get(i)){   
                soreList.add(i,time);     //将记录插入到列表中的指定位置
                if(soreList.size() > n){  
                    soreList.remove(n);  //删除多余的记录
                }
                return;
            }       
        }
        //若当前记录小于列表中的所有记录，则将其加入到列表尾部
        if(soreList.size() < n){  
            soreList.add(time);  //将记录加入到列表尾部
        }
    }

    public void addChess(double rate){

        for(int i=1; i<10 * rate; i++){
            int n = Greenfoot.getRandomNumber(7);
            Actor piece = null;
            switch(n){
                case 0: piece = new Soldier(ChessSide.BLACK);break;
                case 1: if(Greenfoot.getRandomNumber(10)<1) piece = new Chariot(ChessSide.BLACK);break;
                case 2: piece = new Advisor(ChessSide.BLACK);break;
                case 3: if(Greenfoot.getRandomNumber(10)<2)piece = new Cannon(ChessSide.BLACK);break;
                case 4: if(Greenfoot.getRandomNumber(10)<3)piece = new Horse(ChessSide.BLACK);break;
                case 5: piece = new Elephant(ChessSide.BLACK);break;
                case 6: if(Greenfoot.getRandomNumber(10)<3)piece = new Tool();break;
            }
            int x = Greenfoot.getRandomNumber(9);
            while(getObjectsAt(x, 0, Actor.class).size()!=0){
                x = Greenfoot.getRandomNumber(9);
            }
            if(piece != null){
                addObject(piece, x , 0);
            }
        }
    }

    public Score getScore() {
        return score;
    }

    //获取某一方格内的棋子
    public Piece getPieceAt(int x, int y) {
        List<Piece> pieces = getObjectsAt(x, y, Piece.class);
        if (pieces.size() > 0) {
            return pieces.get(0);
        }
        return null;
    }

    //检查某一方格是否有棋子
    public boolean isOccupiedAt(int x, int y) {
        List<Piece> pieces = getObjectsAt(x, y, Piece.class);
        if (pieces.size() > 0) {
            return true;
        }
        return false;
    }

    //获取当前轮到的下棋方
    public ChessSide getCurrentSide() {
        return currentSide;
    }

    //交换下棋方
    public void switchSide() {
        checkGameOver();
        switch (currentSide) {
            case RED:
            currentSide = ChessSide.BLACK;
            break;
            case BLACK:
            currentSide = ChessSide.RED;
            break;
        }
        List<CurrentSide> sides = getObjects(CurrentSide.class);
        sides.get(0).switchSide();
    }

    //检查游戏是否结束
    private void checkGameOver() {
        List<General> generals = getObjects(General.class);
        if (generals.size() != 2) {
            if(generals.get(0).getSide() == ChessSide.RED){
                addObject(new Result(ChessSide.RED), 4, 4);
            }
            else{
                addObject(new Result(ChessSide.BLACK), 4, 4);
            }
            Greenfoot.stop();
        }
    }

    //初始化棋盘，摆放开局时的棋子
    private void initBoard(){

        Horse horse = new Horse(ChessSide.RED);
        addObject(horse, 4, 7);
        /*
        Soldier soldier6 = new Soldier(ChessSide.BLACK);
        addObject(soldier6, 8, 3);
        Soldier soldier7 = new Soldier(ChessSide.BLACK);
        addObject(soldier7, 6, 3);
        Soldier soldier8 = new Soldier(ChessSide.BLACK);
        addObject(soldier8, 4, 3);
        Soldier soldier9 = new Soldier(ChessSide.BLACK);
        addObject(soldier9, 2, 3);
        Soldier soldier10 = new Soldier(ChessSide.BLACK);
        addObject(soldier10, 0, 3);
        Chariot chariot3 = new Chariot(ChessSide.BLACK);
        addObject(chariot3, 8, 0);
        Chariot chariot4 = new Chariot(ChessSide.BLACK);
        addObject(chariot4, 0, 0);
        Elephant elephant3 = new Elephant(ChessSide.BLACK);
        addObject(elephant3, 2, 0);
        Elephant elephant4 = new Elephant(ChessSide.BLACK);
        addObject(elephant4, 6, 0);
        Cannon cannon3 = new Cannon(ChessSide.BLACK);
        addObject(cannon3, 7, 2);
        Cannon cannon4 = new Cannon(ChessSide.BLACK);
        addObject(cannon4, 1, 2);
        General general2 = new General(ChessSide.BLACK);
        addObject(general2, 4, 0);
        Horse horse3 = new Horse(ChessSide.BLACK);
        addObject(horse3, 7, 0);
        Horse horse4 = new Horse(ChessSide.BLACK);
        addObject(horse4, 1, 0);
        Advisor advisor3 = new Advisor(ChessSide.BLACK);
        addObject(advisor3, 3, 0);
        Advisor advisor4 = new Advisor(ChessSide.BLACK);
        addObject(advisor4, 5, 0);
         */
    }
}