import greenfoot.*;  // （World、Actor、GreenfootImage、GreenfootSound、Greenfoot及MouseInfo）
//import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

/**
 * 象棋的相／象。
 */
public class Elephant extends Piece {

    //构造方法，建立新的相／象
    public Elephant(ChessSide chessSide) {
        super(chessSide);
    }

    //取得可以吃的棋子
    public List<Piece> getCapturables() {
        Board board = (Board) getWorld();
        List<Piece> capturables = new ArrayList<Piece>();
        int y0 = -1, y1 = -1;

        if (isBottomSide()) {
            y0 = 6;
            y1 = 8;
        } else {
            y0 = 1;
            //y1 = 3;
            y1 = 8;
        }

        // 左方。
        if (getX() > 1) {
            // 左上方。
            if (getY() > y0) {
                // 没有拐象眼。
                if (!board.isOccupiedAt(getX() - 1, getY() - 1)) {
                    Piece piece = board.getPieceAt(getX() - 2, getY() - 2);
                    if (piece != null) {
                        if (getSide() != piece.getSide()) {
                            capturables.add(piece);
                        }
                    }
                }
            }
            // 左下方。
            if (getY() < y1) {
                // 没有拐象眼。
                if (!board.isOccupiedAt(getX() - 1, getY() + 1)) {
                    Piece piece = board.getPieceAt(getX() - 2, getY() + 2);
                    if (piece != null) {
                        if (getSide() != piece.getSide()) {
                            capturables.add(piece);
                        }
                    }
                }
            }
        }
        // 右方。
        if (getX() < 7) {
            // 右上方。
            if (getY() > y0) {
                // 没有拐象眼。
                if (!board.isOccupiedAt(getX() + 1, getY() - 1)) {
                    Piece piece = board.getPieceAt(getX() + 2, getY() - 2);
                    if (piece != null) {
                        if (getSide() != piece.getSide()) {
                            capturables.add(piece);
                        }
                    }
                }
            }
            // 右下方。
            if (getY() < y1) {
                // 没有拐象眼。
                if (!board.isOccupiedAt(getX() + 1, getY() + 1)) {
                    Piece piece = board.getPieceAt(getX() + 2, getY() + 2);
                    if (piece != null) {
                        if (getSide() != piece.getSide()) {
                            capturables.add(piece);
                        }
                    }
                }
            }
        }
        return capturables;
    }

    //取得可以走的空格
    public List<Point> getMovables() {
        Board board = (Board) getWorld();
        List<Point> movables = new ArrayList<Point>();
        int y0 = -1, y1 = -1;

        if (isBottomSide()) {
            y0 = 6;
            y1 = 8;
        } else {
            y0 = 1;
            y1 = 3;
        }

        // 左方。
        if (getX() > 1) {
            // 左上方。
            if (getY() > y0) {
                // 没有拐象眼。
                if (!board.isOccupiedAt(getX() - 1, getY() - 1)) {
                    if (!board.isOccupiedAt(getX() - 2, getY() - 2)) {
                        movables.add(new Point(getX() - 2, getY() - 2));
                    }
                }
            }
            // 左下方。
            if (getY() < y1) {
                // 没有拐象眼。
                if (!board.isOccupiedAt(getX() - 1, getY() + 1)) {
                    if (!board.isOccupiedAt(getX() - 2, getY() + 2)) {
                        movables.add(new Point(getX() - 2, getY() + 2));
                    }
                }
            }
        }
        // 右方。
        if (getX() < 7) {
            // 右上方。
            if (getY() > y0) {
                // 没有拐象眼。
                if (!board.isOccupiedAt(getX() + 1, getY() - 1)) {
                    if (!board.isOccupiedAt(getX() + 2, getY() - 2)) {
                        movables.add(new Point(getX() + 2, getY() - 2));
                    }
                }
            }
            // 右下方。
            if (getY() < y1) {
                // 没有拐象眼。
                if (!board.isOccupiedAt(getX() + 1, getY() + 1)) {
                    if (!board.isOccupiedAt(getX() + 2, getY() + 2)) {
                        movables.add(new Point(getX() + 2, getY() + 2));
                    }
                }
            }
        }
        return movables;
    }
}