import greenfoot.*;  // （World、Actor、GreenfootImage、GreenfootSound、Greenfoot及MouseInfo）
//import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

/**
 * 象棋的帥／將。
 */
public class General extends Piece {

    //构造方法，建立新的帥／將
    public General(ChessSide chessSide) {
        super(chessSide);
    }

    //取得可以吃的棋子
    public List<Piece> getCapturables() {
        Board board = (Board) getWorld();
        List<Piece> capturables = new ArrayList<Piece>();
        int y0 = -1, y1 = -1;

        if (isBottomSide()) {
            y0 = 7;
            y1 = 9;
        } else {
            y0 = 0;
            y1 = 2;
        }

        // 左边。
        if (getX() > 3) {
            Piece piece = board.getPieceAt(getX() - 1, getY());
            if (piece != null) {
                if (getSide() != piece.getSide()) {
                    capturables.add(piece);
                }
            }
        }
        // 右边。
        if (getX() < 5) {
            Piece piece = board.getPieceAt(getX() + 1, getY());
            if (piece != null) {
                if (getSide() != piece.getSide()) {
                    capturables.add(piece);
                }
            }
        }
        // 上面。
        if (getY() > y0) {
            Piece piece = board.getPieceAt(getX(), getY() - 1);
            if (piece != null) {
                if (getSide() != piece.getSide()) {
                    capturables.add(piece);
                }
            }
        }
        // 下面。
        if (getY() < y1) {
            Piece piece = board.getPieceAt(getX(), getY() + 1);
            if (piece != null) {
                if (getSide() != piece.getSide()) {
                    capturables.add(piece);
                }
            }
        }
        // 将帅不能见面。
        if (isBottomSide()) {
            for (int y = getY() - 1; y >= 0; y--) {
                Piece piece = board.getPieceAt(getX(), y);
                if (piece != null) {
                    if (piece instanceof General && getSide() != piece.getSide()) {
                        capturables.add(piece);
                    }
                    break;
                }
            }
        } else {
            for (int y = getY() + 1; y < 10; y++) {
                Piece piece = board.getPieceAt(getX(), y);
                if (piece != null) {
                    if (piece instanceof General && getSide() != piece.getSide()) {
                        capturables.add(piece);
                    }
                    break;
                }
            }
        }
        return capturables;
    }

    //取得可以走的空格
    public List<Point> getMovables() {
        Board board = (Board) getWorld();
        List<Point> movables = new ArrayList<Point>();
        int y0 = -1, y1 = -1;

        if (isBottomSide()) {
            y0 = 7;
            y1 = 9;
        } else {
            y0 = 0;
            y1 = 2;
        }

        // 左边。
        if (getX() > 3) {
            if (!board.isOccupiedAt(getX() - 1, getY())) {
                movables.add(new Point(getX() - 1, getY()));
            }
        }
        // 右边。
        if (getX() < 5) {
            if (!board.isOccupiedAt(getX() + 1, getY())) {
                movables.add(new Point(getX() + 1, getY()));
            }
        }
        // 上面。
        if (getY() > y0) {
            if (!board.isOccupiedAt(getX(), getY() - 1)) {
                movables.add(new Point(getX(), getY() - 1));
            }
        }
        // 下面。
        if (getY() < y1) {
            if (!board.isOccupiedAt(getX(), getY() + 1)) {
                movables.add(new Point(getX(), getY() + 1));
            }
        }
        return movables;
    }
}