import greenfoot.*;  // （World、Actor、GreenfootImage、GreenfootSound、Greenfoot及MouseInfo）
//import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

/**
 * 象棋的傌／馬。
 */
public class Horse extends Piece {

    //构造方法，建立新的傌／馬。
    public Horse(ChessSide chessSide) {
        super(chessSide);
    }

    //取得可以吃的棋子
    public List<Piece> getCapturables() {
        Board board = (Board) getWorld();
        List<Piece> capturables = new ArrayList<Piece>();

        // 左邊。
        if (getX() > 1) {
            if (!board.isOccupiedAt(getX() - 1, getY())) {
                if (getY() > 0) {
                    Piece piece = board.getPieceAt(getX() - 2, getY() - 1);
                    if (piece != null) {
                        if (getSide() != piece.getSide()) {
                            capturables.add(piece);
                        }
                    }
                }
                if (getY() < 9) {
                    Piece piece = board.getPieceAt(getX() - 2, getY() + 1);
                    if (piece != null) {
                        if (getSide() != piece.getSide()) {
                            capturables.add(piece);
                        }
                    }
                }
            }
        }
        // 右邊。
        if (getX() < 7) {
            if (!board.isOccupiedAt(getX() + 1, getY())) {
                if (getY() > 0) {
                    Piece piece = board.getPieceAt(getX() + 2, getY() - 1);
                    if (piece != null) {
                        if (getSide() != piece.getSide()) {
                            capturables.add(piece);
                        }
                    }
                }
                if (getY() < 9) {
                    Piece piece = board.getPieceAt(getX() + 2, getY() + 1);
                    if (piece != null) {
                        if (getSide() != piece.getSide()) {
                            capturables.add(piece);
                        }
                    }
                }
            }
        }
        // 上面。
        if (getY() > 1) {
            if (!board.isOccupiedAt(getX(), getY() - 1)) {
                if (getX() > 0) {
                    Piece piece = board.getPieceAt(getX() - 1, getY() - 2);
                    if (piece != null) {
                        if (getSide() != piece.getSide()) {
                            capturables.add(piece);
                        }
                    }
                }
                if (getX() < 8) {
                    Piece piece = board.getPieceAt(getX() + 1, getY() - 2);
                    if (piece != null) {
                        if (getSide() != piece.getSide()) {
                            capturables.add(piece);
                        }
                    }
                }
            }
        }
        // 下面。
        if (getY() < 8) {
            if (!board.isOccupiedAt(getX(), getY() + 1)) {
                if (getX() > 0) {
                    Piece piece = board.getPieceAt(getX() - 1, getY() + 2);
                    if (piece != null) {
                        if (getSide() != piece.getSide()) {
                            capturables.add(piece);
                        }
                    }
                }
                if (getX() < 8) {
                    Piece piece = board.getPieceAt(getX() + 1, getY() + 2);
                    if (piece != null) {
                        if (getSide() != piece.getSide()) {
                            capturables.add(piece);
                        }
                    }
                }
            }
        }
        
        return capturables;
    }

    //取得可以走的空格
    public List<Point> getMovables() {
        
        Board board = (Board) getWorld();
        List<Point> movables = new ArrayList<Point>();

        // 左邊。
        if (getX() > 1) {
            if (!board.isOccupiedAt(getX() - 1, getY())) {
                if (getY() > 0) {
                    if (!board.isOccupiedAt(getX() - 2, getY() - 1)) {
                        //Greenfoot.playSound("fail.wav");
                        movables.add(new Point(getX() - 2, getY() - 1));
                        
                    }
                }
                if (getY() < 9) {
                    if (!board.isOccupiedAt(getX() - 2, getY() + 1)) {
                        movables.add(new Point(getX() - 2, getY() + 1));
                    }
                }
            }
        }
        
        // 右邊。
        if (getX() < 7) {
            if (!board.isOccupiedAt(getX() + 1, getY())) {
                if (getY() > 0) {
                    if (!board.isOccupiedAt(getX() + 2, getY() - 1)) {
                        movables.add(new Point(getX() + 2, getY() - 1));
                    }
                }
                if (getY() < 9) {
                    if (!board.isOccupiedAt(getX() + 2, getY() + 1)) {
                        movables.add(new Point(getX() + 2, getY() + 1));
                    }
                }
            }
        }
        // 上面。
        if (getY() > 1) {
            if (!board.isOccupiedAt(getX(), getY() - 1)) {
                if (getX() > 0) {
                    if (!board.isOccupiedAt(getX() - 1, getY() - 2)) {
                        movables.add(new Point(getX() - 1, getY() - 2));
                    }
                }
                if (getX() < 8) {
                    if (!board.isOccupiedAt(getX() + 1, getY() - 2)) {
                        movables.add(new Point(getX() + 1, getY() - 2));
                    }
                }
            }
        }
        // 下面。
        if (getY() < 8) {
            if (!board.isOccupiedAt(getX(), getY() + 1)) {
                if (getX() > 0) {
                    if (!board.isOccupiedAt(getX() - 1, getY() + 2)) {
                        movables.add(new Point(getX() - 1, getY() + 2));
                    }
                }
                if (getX() < 8) {
                    if (!board.isOccupiedAt(getX() + 1, getY() + 2)) {
                        movables.add(new Point(getX() + 1, getY() + 2));
                    }
                }
            }
        }
        
        return movables;
    }
}