import greenfoot.*;  // （World、Actor、GreenfootImage、GreenfootSound、Greenfoot及MouseInfo）
//import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

/**
 * 象棋棋子类
 */
public abstract class Piece extends Actor {

    private ChessSide side = null;  //棋子所属的阵营，红方还是黑方
    private boolean isBottomSide = true;  //棋子阵地是否在棋盘下方
    private boolean isDragging = false;  //是否正在被鼠标拖拽中
    private int fromX = -1;  //棋子原来位置的横坐标
    private int fromY = -1;  //棋子原来位置的纵坐标
    private boolean selected = false;

    //构造方法，建立新的棋子
    public Piece(ChessSide chessSide) {
        side = chessSide;
        setImage(getClass().getName().toLowerCase() + "-" + chessSide + ".png");
    }

    //棋子加进棋盘后要执行的动作
    protected void addedToWorld(World world) {            
        if (side == ChessSide.BLACK) {
            GreenfootImage image = getImage();
            image.rotate(180);
            isBottomSide = false;
        }
    }

    //游戏循环，每一回合执行一次
    public void act() {
        /*Board board = (Board) getWorld();

        if (side == board.getCurrentSide()) {
        //开始拖拽棋子，标示可移动的位置

        if (!isDragging && Greenfoot.mouseDragged(this)) {
        isDragging = true;
        fromX = getX();
        fromY = getY();
        tagAvailableMoves();
        }
        //棋子在拖拽中，只能拖动到可走棋的位置
        if (Greenfoot.mouseDragged(this)) {
        MouseInfo mouse = Greenfoot.getMouseInfo();
        List<Hint> hints = board.getObjectsAt(mouse.getX(), mouse.getY(), Hint.class);
        if (hints.size() > 0) {
        setLocation(mouse.getX(), mouse.getY());
        }
        }
        //拖拽结束，走棋完成
        if (Greenfoot.mouseDragEnded(this)) {
        isDragging = false;
        List<Hint> hints = board.getObjects(Hint.class);
        board.removeObjects(hints);
        //若棋子的原始位置和目标位置不同，则更新棋子位置
        if (getX() != fromX || getY() != fromY) {
        List<Piece> pieces = board.getObjectsAt(getX(), getY(), Piece.class);
        pieces.remove(this);
        board.removeObjects(pieces);   //移除被吃掉的棋子
        //board.switchSide();            //换到对方走棋

        if (getY() == 5){
        updateLocation(2);
        }else if (getY() == 6){
        updateLocation(1);
        }
        checkBlackEat();
        }
        }*/
        if(Board.gameover) return;

        Board board = (Board) getWorld();

        if (side == board.getCurrentSide()) {
            if (Greenfoot.mouseClicked(this)) {
                if(!selected){
                    fromX = getX();
                    fromY = getY();
                    tagAvailableMoves();
                    selected = true;
                }
            }
            boolean moved = false;
            List<Hint> hints = board.getObjects(Hint.class);
            for (Hint hint: hints){
                if(Greenfoot.mouseClicked(hint)){
                    setLocation(hint.getX(), hint.getY());
                    moved = true;
                    break;
                }
            }
            if(moved){
                selected = false;
                board.removeObjects(hints);
                List<Piece> pieces = board.getObjectsAt(getX(), getY(), Piece.class);
                if(pieces.size() > 1){
                    ((Board)getWorld()).getScore().increase(5);
                }
                pieces.remove(this);
                board.removeObjects(pieces); 

                if (getY() == 5){
                    updateLocation(2);
                    Tool.count --;
                }else if (getY() == 6){
                    updateLocation(1);
                    Tool.count --;
                }
                else if(getY() < 5){
                    int n = 7-getY();
                    for(int i=0; i< n; i++){
                        //System.out.println(getY());
                        updateLocation(1);
                    }
                    Tool.count --;
                }

                checkBlackEat();
                if(Tool.count <= 0){
                    board.addObject(new Horse(ChessSide.RED), getX(), getY());
                    board.removeObject(this);
                }
            }
        }
    }

    public void updateLocation(int offset){
        List pieces = getWorld().getObjects(Piece.class);
        for(Object piece: pieces){      
            int x = ((Piece)piece).getX();
            int y = ((Piece)piece).getY();
            if (y + offset > 9){
                getWorld().removeObject((Piece)piece);
            }else{
                ((Piece)piece).setLocation(x, y + offset);
            }
        }
        List tools = getWorld().getObjects(Tool.class);
        for(Object tool: tools){      
            int x = ((Tool)tool).getX();
            int y = ((Tool)tool).getY();
            if (y + offset > 9){
                getWorld().removeObject((Tool)tool);
            }else{
                ((Tool)tool).setLocation(x, y + offset);
            }
        }
        Greenfoot.playSound("fall.wav");
        ((Board)getWorld()).getScore().increase(offset * 10); 
        ((Board)getWorld()).addChess(offset * 0.2);
    }

    public void checkBlackEat(){
        List pieces = getWorld().getObjects(Piece.class);
        for(Object p: pieces){
            Piece piece = (Piece)p;
            if(piece.getSide() == ChessSide.RED) continue;
            List<Piece> ls = piece.getCapturables();
            if(ls.size() != 0){
                Board board = (Board)getWorld();
                Piece target = ls.get(0);
                board.addObject(new Hint(), piece.getX(), piece.getY());
                Board.gameover = true;
                board.addObject(new Animation(piece, target.getX(), target.getY()), 0, 0);

                return;
                //piece.setLocation(target.getX(), target.getY());                            
                //board.removeObject(target);
                // board.gameOver();
            }
        }
    }

    //取得棋子所属的阵营
    public ChessSide getSide() {
        return side;
    }

    //检查棋子阵地是否在棋盘下方
    public boolean isBottomSide() {
        return isBottomSide;
    }

    //标识下一步可以走的位置
    public void tagAvailableMoves() {
        Board board = (Board) getWorld();
        int x = getX(), y = getY();
        //board.gameOver();
        //board.addObject(new Hint(), x, y);
        for (Piece target : getCapturables()) {
            board.addObject(new Hint(), target.getX(), target.getY());
        }
        for (Point target : getMovables()) {
            board.addObject(new Hint(), target.x, target.y);
        }

        // 重新加入自己，把自己疊在標示的上方。
        //board.removeObject(this);
        //board.addObject(this, x, y);
    }

    //取得可以吃掉的棋子
    public abstract List<Piece> getCapturables();

    //取得可以移动的方格
    public abstract List<Point> getMovables();

}